package router

import (
	"fonchain-fiee/pkg/middleware"
	"fonchain-fiee/pkg/service"
	"fonchain-fiee/pkg/service/bundle"

	"github.com/gin-gonic/gin"
)

func BundleOrderRouter(r *gin.RouterGroup) {

	r.POST("/stripe-pay/callback", bundle.StripeCheckoutSessionWebhook)
	r.POST("/antom/callback", bundle.AntomWebhook) // Antom回调

	bundleOrderRoute := r.Group("bundle-order")
	bundleOrderRoute.Use(middleware.CheckLogin(service.AccountFieeProvider))
	//bundleOrderWebRoute := r.Group("bundle-order")
	//bundleOrderWebRoute.Use(middleware.CheckWebLogin(service.AccountProvider))

	// 套餐
	{
		bundleOrderClientWebRoute := bundleOrderRoute.Group("common/web")
		{
			bundleOrderClientWebRoute.POST("bundle-order-list", bundle.OrderRecordsList)
		}
		bundleOrderClientAppRoute := bundleOrderRoute.Group("common/app")
		{
			bundleOrderClientAppRoute.POST("bundle-order-list", bundle.OrderRecordsList)
		}

		//bundleOrderWebRoute := bundleOrderRoute.Group("web")
		{
			bundleOrderRoute.POST("financial-confirm", bundle.UpdateFinancialConfirmationStatus)
			bundleOrderRoute.POST("order-export", bundle.ExportOrderInfo)
		}

		bundleOrderAppRoute := bundleOrderRoute.Group("app")
		{
			bundleOrderAppRoute.POST("order-signature", bundle.CreateBundleOrderSignature)
			bundleOrderAppRoute.POST("update-pay", bundle.UpdateBundleOrderStatusPaid)
			bundleOrderAppRoute.POST("order-detail", bundle.OrderRecordsDetail)

			// stripe
			//bundleOrderAppRoute.POST("create-stripe-pay", bundle.CreateStripeCheckoutSession)	// stripe支付
			bundleOrderAppRoute.POST("create-stripe-pay", bundle.CreateAntomPay) // Antom国际支付
		}

	}

}
